#
# Copyright (c) 2008-2015 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

__docformat__ = 'restructuredtext'


# import standard library

# import interfaces
from pyams_content.component.gallery.interfaces import IGallery, IGalleryTarget, \
    GALLERY_CONTAINER_KEY, IGalleryRenderer
from pyams_content.shared.common.interfaces import IWfSharedContent
from pyams_form.interfaces.form import IFormContextPermissionChecker
from zope.annotation.interfaces import IAnnotations
from zope.lifecycleevent.interfaces import IObjectAddedEvent, IObjectModifiedEvent, IObjectRemovedEvent
from zope.location.interfaces import ISublocations
from zope.traversing.interfaces import ITraversable

# import packages
from pyams_catalog.utils import index_object
from pyams_utils.adapter import adapter_config, ContextAdapter
from pyams_utils.container import BTreeOrderedContainer
from pyams_utils.request import check_request
from pyams_utils.traversing import get_parent
from pyams_utils.vocabulary import vocabulary_config
from pyramid.events import subscriber
from pyramid.threadlocal import get_current_registry
from zope.interface import implementer
from zope.lifecycleevent import ObjectCreatedEvent, ObjectModifiedEvent
from zope.location import locate
from zope.schema.fieldproperty import FieldProperty
from zope.schema.vocabulary import SimpleVocabulary, SimpleTerm


#
# Galleries container
#

@implementer(IGallery)
class Gallery(BTreeOrderedContainer):
    """Gallery persistent class"""

    title = FieldProperty(IGallery['title'])
    description = FieldProperty(IGallery['description'])
    renderer = FieldProperty(IGallery['renderer'])

    last_id = 1

    def append(self, value, notify=True):
        key = str(self.last_id)
        if not notify:
            # pre-locate gallery item to avoid multiple notifications
            locate(value, self.key)
        self[key] = value
        self.last_id += 1
        if not notify:
            # make sure that gallery item is correctly indexed
            index_object(value)

    def get_visible_images(self):
        return [image for image in self.values() if image.visible]


@adapter_config(context=IGalleryTarget, provides=IGallery)
def gallery_factory(target):
    """Galleries container factory"""
    annotations = IAnnotations(target)
    gallery = annotations.get(GALLERY_CONTAINER_KEY)
    if gallery is None:
        gallery = annotations[GALLERY_CONTAINER_KEY] = Gallery()
        get_current_registry().notify(ObjectCreatedEvent(gallery))
        locate(gallery, target, '++gallery++')
    return gallery


@adapter_config(name='gallery', context=IGalleryTarget, provides=ITraversable)
class GalleryContainerNamespace(ContextAdapter):
    """++gallery++ namespace traverser"""

    def traverse(self, name, furtherpath=None):
        return IGallery(self.context)


@adapter_config(name='gallery', context=IGalleryTarget, provides=ISublocations)
class GalleryContainerSublocations(ContextAdapter):
    """Galleries container sublocations"""

    def sublocations(self):
        return IGallery(self.context).values()


@adapter_config(context=IGallery, provides=IFormContextPermissionChecker)
class GalleryPermissionChecker(ContextAdapter):
    """Gallery permission checker"""

    @property
    def edit_permission(self):
        content = get_parent(self.context, IWfSharedContent)
        return IFormContextPermissionChecker(content).edit_permission


@subscriber(IObjectAddedEvent, context_selector=IGallery)
def handle_added_gallery(event):
    """Handle added gallery"""
    content = get_parent(event.object, IWfSharedContent)
    if content is not None:
        get_current_registry().notify(ObjectModifiedEvent(content))


@subscriber(IObjectModifiedEvent, context_selector=IGallery)
def handle_modified_gallery(event):
    """Handle modified gallery"""
    content = get_parent(event.object, IWfSharedContent)
    if content is not None:
        get_current_registry().notify(ObjectModifiedEvent(content))


@subscriber(IObjectRemovedEvent, context_selector=IGallery)
def handle_removed_gallery(event):
    """Handle removed gallery"""
    content = get_parent(event.object, IWfSharedContent)
    if content is not None:
        get_current_registry().notify(ObjectModifiedEvent(content))


@vocabulary_config(name='PyAMS gallery renderers')
class GalleryRendererVocabulary(SimpleVocabulary):
    """Gallery renderer utilities vocabulary"""

    def __init__(self, context=None):
        request = check_request()
        translate = request.localizer.translate
        registry = request.registry
        context = Gallery()
        terms = [SimpleTerm(name, title=translate(adapter.label))
                 for name, adapter in sorted(registry.getAdapters((context, request), IGalleryRenderer),
                                             key=lambda x: x[1].weight)]
        super(GalleryRendererVocabulary, self).__init__(terms)
